/**
 * Spring, Hibernate y modelos anémicos
 * Copyright 2010 Autentia Real Business Solutions S.L.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.autentia.tutorial.modelos.anemicos;

import java.util.List;

import javax.inject.Inject;

import org.hibernate.SessionFactory;
import org.hibernate.classic.Session;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

@Repository
@Transactional(readOnly = true)
class DomainObjectRepository {

	private static final String FIND_BY_EXACT_NAME = "from " + DomainObject.class.getSimpleName()
			+ " do where do.name=:name";

	private SessionFactory sessionFactory;

	DomainObjectRepository() {
		// Para uso de CGLIB
	}

	@Inject
	DomainObjectRepository(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@SuppressWarnings("unchecked")
	List<DomainObject> findByExact(String name) {
		return getCurrentSession().createQuery(FIND_BY_EXACT_NAME).setString("name", name).list();
	}

	DomainObject getBy(Long id) {
		return (DomainObject)getCurrentSession().get(DomainObject.class, id);
	}

	Session getCurrentSession() {
		return sessionFactory.getCurrentSession();
	}

	DomainObject loadBy(Long id) {
		return (DomainObject)getCurrentSession().load(DomainObject.class, id);
	}

	@Transactional(readOnly = false)
	void save(DomainObject domainObject) {
		getCurrentSession().save(domainObject);
	}
}
