/**
 * Spring, Hibernate y modelos anémicos
 * Copyright 2010 Autentia Real Business Solutions S.L.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.autentia.tutorial.modelos.anemicos;

import static org.junit.Assert.*;

import java.util.List;

import javax.inject.Inject;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.transaction.annotation.Transactional;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration({ "classpath:applicationContext.xml", "classpath:applicationContext-infrastructure-test.xml" })
public class DomainObjectTest {

	@Inject
	private DomainObjectRepository domainObjectRepository;

	private Long persistedDomainObjectId;

	@Inject
	private ServiceToInject serviceToInject;

	@Test
	@Transactional
	public void a_dependency_is_injected_to_my_domain_objects_when_find_from_hibernate() {
		insertDataInTheDatabase();
		
		List<DomainObject> domainObjects = domainObjectRepository.findByExact("Autentia");
		assertEquals(3, domainObjects.size());
		for (DomainObject domainObject : domainObjects) {
			assertInjectedDependencyIn(domainObject);
		}
	}

	@Test
	@Transactional
	public void a_dependency_is_injected_to_my_domain_objects_when_get_from_hibernate() {
		insertDataInTheDatabase();
		
		assertInjectedDependencyIn(domainObjectRepository.getBy(persistedDomainObjectId));
	}

	@Test
	@Transactional
	public void a_dependency_is_injected_to_my_domain_objects_when_load_from_hibernate() {
		insertDataInTheDatabase();
		
		assertInjectedDependencyIn(domainObjectRepository.loadBy(persistedDomainObjectId));
	}
	
	@Test
	public void a_dependency_is_injected_to_my_domain_objects_when_new_is_done() {
		DomainObject domainObject = new DomainObject("Autentia");
		assertInjectedDependencyIn(domainObject);
	}

	private void assertInjectedDependencyIn(DomainObject domainObject) {
		assertNotNull(domainObject);
		
		final Object injectedDependency = domainObject.getInjectedDependency();

		assertNotNull(injectedDependency);
		assertSame(serviceToInject, injectedDependency);
	}

	private void insertDataInTheDatabase() {
		for (int i = 0;  i < 3; i++) {
			final DomainObject domainObject = new DomainObject("Autentia");
			domainObjectRepository.save(domainObject);
			persistedDomainObjectId = domainObject.getId();
		}		
	}
}
